using System;
using System.IO;                         // Allows for Input/Output to both screen and files
using System.Xml;                        // Gives us the ability to work with Xml objects
using System.Data.Odbc;                  // Provides ODBC Connectivity
using System.Web.Security;               // Provides access to the Hashing function that encrypts the PW for the Xml file


public class XmlGenerator
{
  // Private Member Variables (Arguments)
  private string m_argXmlFile; 
  private string m_argDsn;
  private string m_argSqlFile;
  private string m_argFields;
  private string m_argRole;
  
  // Private Member Variables (Other)
  private OdbcConnection m_conn;
  private OdbcDataReader m_reader;
  private string m_strSql;
  
  
  public XmlGenerator(string _strXmlFile, string _strDsn, string _strSqlFile, string _strFields, string _strRole)
  {
    m_argXmlFile = _strXmlFile;
    m_argDsn = _strDsn;
    m_argSqlFile = _strSqlFile;
    m_argFields = _strFields;
    m_argRole = _strRole;
  }
  
  public void GetArguments(string[] _args)
  {
    for (int i = 0; i < _args.Length; i+=2)    
    {
      switch (_args[i].ToLower())
      {
        case "-xmlfile":
          m_argXmlFile = _args[i+1];
          break;
        
        case "-dsn":
          m_argDsn = _args[i+1];
          break;
          
        case "-sqlfile":
          m_argSqlFile = _args[i+1];
          break;
          
        case "-fields":
          m_argFields = _args[i+1];
          break;
          
        case "-role":
          m_argRole = _args[i+1];
          break;
      }
    }  
  }
  
  public bool ReadSqlFile()
  {
    try
    {
      TextReader tr = System.IO.File.OpenText(m_argSqlFile);
      m_strSql = tr.ReadToEnd();
      tr.Close();
      return true;
    }
    catch
    {
      return false;
    } 
  }
  
  public bool ConnectToDatabase()
  {
    // Try to make a connection to the database    
    try
    {
      string strConnString = "DSN=" + m_argDsn;
      m_conn = new OdbcConnection(strConnString);
      m_conn.Open();
      return true;
    }
    catch
    {
      return false;     
    }    
  }
  
  public bool ExecuteQuery()
  {
    // Try to execute a query on the database    
    try
    {
      OdbcCommand cmd = new OdbcCommand(m_strSql, m_conn);
      m_reader = cmd.ExecuteReader();
      return true;
    }
    catch
    {
      return false;
    }    
  }
  
  public bool CreateXmlFile()
  {
    string[] fields = m_argFields.Split(',');
    
    try
    {
      XmlTextWriter writer = new XmlTextWriter(m_argXmlFile, null);
      writer.Formatting = Formatting.Indented;
      writer.Indentation = 2;
      
      writer.WriteStartDocument();
      writer.WriteStartElement("Customers");
      writer.WriteAttributeString("role", m_argRole);
      
      while (m_reader.Read())
      {
        string fieldName = "", fieldValue = "";
        
        writer.WriteStartElement("Customer");
        for (int i = 0; i < fields.Length; i++)
        {
          fieldValue = m_reader.GetValue(i).ToString();
          string[] fieldOps = fields[i].Split('/');          
          fieldName = fieldOps[0];
          for (int j = 1; j < fieldOps.Length; j++)          
          {
            switch (fieldOps[j].ToLower())
            {
              case "t":
                fieldValue = fieldValue.Trim();
                break;
                
              case "u":
                fieldValue = fieldValue.ToUpper();
                break;
                
              case "l":
                fieldValue = fieldValue.ToLower();
                break;
                
              case "h":
                fieldValue = FormsAuthentication.HashPasswordForStoringInConfigFile(fieldValue, "MD5");
                break;
            }
          }
          writer.WriteElementString(fieldName, fieldValue);                  
        }        
        writer.WriteEndElement();
      }
      
      writer.WriteEndElement();
      writer.WriteEndDocument();
      
      writer.Flush();
      writer.Close();
      
      m_reader.Close();
      m_conn.Close();
      return true;      
    }
    catch
    {
      return false;
    }
  }
}      


public class GenerateXmlFile
{
    
  // main method - program point of entry
  public static int Main(string[] args)
  {
    string strXmlFile = "customers.xml"; 
    string strDsn = "ups";
    string strSqlFile = "customers.sql";
    string strFields = "UserId,Password,Company,Address,City,State,Zip";
    string strRole = "Customers";
    
    XmlGenerator xg = new XmlGenerator(strXmlFile, strDsn, strSqlFile, strFields, strRole);
    
    xg.GetArguments(args);
    
    if (!xg.ReadSqlFile())
    {  
      Console.WriteLine("Error reading SQL text file.");
      return 1;
    }
    
    if (!xg.ConnectToDatabase())
    {
      Console.WriteLine("Error connecting to database.");
      return 2;
    }
      
    if (!xg.ExecuteQuery())
    {
      Console.WriteLine("Error executing SQL query.");
      return 3;
    }
    
    if (!xg.CreateXmlFile())
    {
      Console.WriteLine("Error generating XML file.");
      return 4;
    }    
    
    // Everything worked!
    Console.WriteLine("Xml file generated successfully.");
    return 0;
  }
}
